/* Terraform - (C) 1997-1999 Robert Gasch (Robert_Gasch@peoplesoft.com)
 *  - www.peoplesoft.com/peoplepages/g/robert_gasch/terraform/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */


#include "TFDialogGaussianHill.h"
#include "GlobalSanityCheck.h"
#include "GlobalTrace.h"
#include "MathTrig.h"
#include "GuiDialogOAC.h"


/*
 *  constructor: initialize all data members 
 */
TFDialogGaussianHill::TFDialogGaussianHill (HeightField *HF, HeightFieldDraw *HFD)
	        : TFPreviewDialog (HF, HFD, "Terraform Gaussian Hill Dialog", 
					"Parameters", 6, 5),
		  d_vBoxList (TRUE, 0),
		  d_hBoxCX (TRUE, 5),
		  d_hBoxCY (TRUE, 5),
		  d_hBoxRadius (TRUE, 5),
		  d_hBoxScaleFactor (TRUE, 5),
		  d_hBoxRadFac (TRUE, 5),
		  d_hBoxSmoothFactor (TRUE, 5),
		  d_hBoxDeltaScale (TRUE, 5),
		  d_optionsFrame ("Gaussian Hill Options")
{
	GlobalTrace::trace (GlobalTrace::TRACE_FLOW, "+++ TFDialogGaussianHill\n");

	d_lblCX = new Gtk_Label ("Center X");
	d_lblCY = new Gtk_Label ("Center Y");
	d_lblRadius = new Gtk_Label ("Hill Radius");
	d_lblScaleFactor = new Gtk_Label ("Scale Factor");
	d_lblRadFac = new Gtk_Label ("Radius Factor");
	d_lblSmoothFactor = new Gtk_Label ("Smoothing Factor");
	d_lblDeltaScale = new Gtk_Label ("Delta Scale Factor");

	d_adjCX = new Gtk_Adjustment (0.5, 0, 1, 0.01);
	d_adjCY = new Gtk_Adjustment (0.5, 0, 1, 0.01);
	d_adjRadius = new Gtk_Adjustment (0.5, 0, 1, 0.01);
	d_adjScaleFactor = new Gtk_Adjustment (1.5, 1, 2, 0.01);
	d_adjRadFac = new Gtk_Adjustment (0.33, 0, 1, 0.01);
	d_adjSmoothFactor = new Gtk_Adjustment (0.5, 0, 1, 0.01);
	d_adjDeltaScale = new Gtk_Adjustment (0.1, 0, 0.5, 0.01);

	d_hsCX = new Gtk_HScale (*d_adjCX);
	d_hsCY = new Gtk_HScale (*d_adjCY);
	d_hsRadius = new Gtk_HScale (*d_adjRadius);
	d_hsScaleFactor = new Gtk_HScale (*d_adjScaleFactor);
	d_hsRadFac = new Gtk_HScale (*d_adjRadFac);
	d_hsSmoothFactor = new Gtk_HScale (*d_adjSmoothFactor);
	d_hsDeltaScale = new Gtk_HScale (*d_adjDeltaScale);

	buildDialogWindow ();
	this->setHFobjs (HF, HFD);
	sprintf (this->d_winTitle, "Gaussian Hill: %s", d_HF->getName());
	this->set_title (this->d_winTitle);
	this->iterateEvents ();
	updatePreviewCallback ();

	connect_to_method (d_adjCX->value_changed, this, &TFDialogGaussianHill::updatePreviewCallback);
	connect_to_method (d_adjCY->value_changed, this, &TFDialogGaussianHill::updatePreviewCallback);
	connect_to_method (d_adjRadius->value_changed, this, &TFDialogGaussianHill::updatePreviewCallback);
	connect_to_method (d_adjScaleFactor->value_changed, this, &TFDialogGaussianHill::updatePreviewCallback);
	connect_to_method (d_adjRadFac->value_changed, this, &TFDialogGaussianHill::updatePreviewCallback);
	connect_to_method (d_adjSmoothFactor->value_changed, this, &TFDialogGaussianHill::updatePreviewCallback);
	connect_to_method (d_adjDeltaScale->value_changed, this, &TFDialogGaussianHill::updatePreviewCallback);
}


/*
 *  destructor: clean up 
 */
TFDialogGaussianHill::~TFDialogGaussianHill ()
{
	delete d_lblCX;
	delete d_lblCY;
	delete d_lblRadius;
	delete d_lblScaleFactor;
	delete d_lblRadFac;
	delete d_lblSmoothFactor;
	delete d_lblDeltaScale;

	delete d_adjCX;
	delete d_adjCY;
	delete d_adjRadius;
	delete d_adjScaleFactor;
	delete d_adjRadFac;
	delete d_adjSmoothFactor;
	delete d_adjDeltaScale;

	delete d_hsCX;
	delete d_hsCY;
	delete d_hsRadius;
	delete d_hsScaleFactor;
	delete d_hsRadFac;
	delete d_hsSmoothFactor;
	delete d_hsDeltaScale;

	GlobalTrace::trace (GlobalTrace::TRACE_FLOW, "--- TFDialogGaussianHill\n");
}


/*
 *  insertOptions: fill the dialog's VBox
 */
void TFDialogGaussianHill::insertOptions ()
{
	d_optionsFrame.set_shadow_type (GTK_SHADOW_ETCHED_IN);
	d_optionsFrame.set_border_width (5);

	d_hBoxCX.pack_start (*d_lblCX, TRUE, TRUE, GuiDialogOAC::s_HBOff);
	d_lblCX->show ();
	d_hsCX->set_update_policy (GTK_UPDATE_CONTINUOUS);
	d_hsCX->set_digits (2);
	d_hsCX->set_draw_value (TRUE);
	d_hBoxCX.pack_end (*d_hsCX, TRUE, TRUE, GuiDialogOAC::s_HBOff);
	d_hsCX->show ();
	d_vBoxList.pack_start (d_hBoxCX, TRUE, TRUE, GuiDialogOAC::s_VBOff);
	d_hBoxCX.show ();

	d_hBoxCY.pack_start (*d_lblCY, TRUE, TRUE, GuiDialogOAC::s_HBOff);
	d_lblCY->show ();
	d_hsCY->set_update_policy (GTK_UPDATE_CONTINUOUS);
	d_hsCY->set_digits (2);
	d_hsCY->set_draw_value (TRUE);
	d_hBoxCY.pack_end (*d_hsCY, TRUE, TRUE, GuiDialogOAC::s_HBOff);
	d_hsCY->show ();
	d_vBoxList.pack_start (d_hBoxCY, TRUE, TRUE, GuiDialogOAC::s_VBOff);
	d_hBoxCY.show ();

	d_hBoxRadius.pack_start (*d_lblRadius, TRUE, TRUE, GuiDialogOAC::s_HBOff);
	d_lblRadius->show ();
	d_hsRadius->set_update_policy (GTK_UPDATE_CONTINUOUS);
	d_hsRadius->set_digits (2);
	d_hsRadius->set_draw_value (TRUE);
	d_hBoxRadius.pack_end (*d_hsRadius, TRUE, TRUE, GuiDialogOAC::s_HBOff);
	d_hsRadius->show ();
	d_vBoxList.pack_start (d_hBoxRadius, TRUE, TRUE, GuiDialogOAC::s_VBOff);
	d_hBoxRadius.show ();

	d_vBoxList.pack_start (d_hSep);
	d_hSep.show ();

	d_hBoxScaleFactor.pack_start (*d_lblScaleFactor, TRUE, TRUE, GuiDialogOAC::s_HBOff);
	d_lblScaleFactor->show ();
	d_hsScaleFactor->set_update_policy (GTK_UPDATE_CONTINUOUS);
	d_hsScaleFactor->set_digits (2);
	d_hsScaleFactor->set_draw_value (TRUE);
	d_hBoxScaleFactor.pack_end (*d_hsScaleFactor, TRUE, TRUE, GuiDialogOAC::s_HBOff);
	d_hsScaleFactor->show ();
	d_vBoxList.pack_start (d_hBoxScaleFactor, TRUE, TRUE, GuiDialogOAC::s_VBOff);
	d_hBoxScaleFactor.show ();

	d_hBoxRadFac.pack_start (*d_lblRadFac, TRUE, TRUE, GuiDialogOAC::s_HBOff);
	d_lblRadFac->show ();
	d_hsRadFac->set_update_policy (GTK_UPDATE_CONTINUOUS);
	d_hsRadFac->set_digits (2);
	d_hsRadFac->set_draw_value (TRUE);
	d_hBoxRadFac.pack_end (*d_hsRadFac, TRUE, TRUE, GuiDialogOAC::s_HBOff);
	d_hsRadFac->show ();
	d_vBoxList.pack_start (d_hBoxRadFac, TRUE, TRUE, GuiDialogOAC::s_VBOff);
	d_hBoxRadFac.show ();

	d_hBoxSmoothFactor.pack_start (*d_lblSmoothFactor, TRUE, TRUE, GuiDialogOAC::s_HBOff);
	d_lblSmoothFactor->show ();
	d_hsSmoothFactor->set_update_policy (GTK_UPDATE_CONTINUOUS);
	d_hsSmoothFactor->set_digits (2);
	d_hsSmoothFactor->set_draw_value (TRUE);
	d_hBoxSmoothFactor.pack_end (*d_hsSmoothFactor, TRUE, TRUE, GuiDialogOAC::s_HBOff);
	d_hsSmoothFactor->show ();
	d_vBoxList.pack_start (d_hBoxSmoothFactor, TRUE, TRUE, GuiDialogOAC::s_VBOff);
	d_hBoxSmoothFactor.show ();

	d_hBoxDeltaScale.pack_start (*d_lblDeltaScale, TRUE, TRUE, GuiDialogOAC::s_HBOff);
	d_lblDeltaScale->show ();
	d_hsDeltaScale->set_update_policy (GTK_UPDATE_CONTINUOUS);
	d_hsDeltaScale->set_digits (2);
	d_hsDeltaScale->set_draw_value (TRUE);
	d_hBoxDeltaScale.pack_end (*d_hsDeltaScale, TRUE, TRUE, GuiDialogOAC::s_HBOff);
	d_hsDeltaScale->show ();
	d_vBoxList.pack_start (d_hBoxDeltaScale, TRUE, TRUE, GuiDialogOAC::s_VBOff);
	d_hBoxDeltaScale.show ();

	d_optionsFrame.add (d_vBoxList);
	this->d_table->attach (d_optionsFrame, 3, 6, 1, 4);
        d_vBoxList.show ();
	d_optionsFrame.show ();
        this->d_baseFrame->show ();
        this->get_vbox()->show ();
}



void TFDialogGaussianHill::buildDialogWindow ()
{
	this->set_usize (450, 350);
	this->set_title (this->d_winTitle);

	this->get_vbox()->set_border_width (2);

	this->setupVBox ();
	this->insertPreview ();
	insertOptions ();
	this->fillActionArea ();

	connect_to_method (d_drawArea->button_press_event, this, 
			&TFDialogGaussianHill::daButtonPressedCallback);
	
	this->show ();
}


/*
 *  buttonCallbackApply: process Apply button
 */
void TFDialogGaussianHill::buttonCallbackApply ()
{
	float		cx, cy, sf, radfac, radius, smooth, deltascale; 

	cx = d_adjCX->get_value ();
	cy = d_adjCY->get_value ();
	radius= d_adjRadius->get_value (); 
        sf = d_adjScaleFactor->get_value ();
	radfac = d_adjRadFac->get_value (); 
	smooth = d_adjSmoothFactor->get_value ();
	deltascale = d_adjDeltaScale->get_value ();

	SanityCheck::bailout ((!d_HFO), "d_HFO==NULL", "TFDialogGaussianHill::buttonCallbackApply");
	d_HFO->gaussianHill (cx, cy, radius, radfac, sf, smooth, deltascale);
	d_HFD->draw ();

	// only do this if window stays open 
	if (d_applyHit)
		{
		this->setHFobjs (d_HF, d_HFD);
		updatePreviewCallback ();
		}

	this->d_applyHit = TRUE;
}


/*
 *  updatePreviewCallback: update the preview after a slider has been updated.
 */
void TFDialogGaussianHill::updatePreviewCallback ()
{
	if (!d_cbUsePreview.get_active())
		return;

	float		cx, cy, radius, sf, radfac, smooth, deltascale; 

	cx = d_adjCX->get_value ();
	cy = d_adjCY->get_value ();
	radius= d_adjRadius->get_value (); 
        sf = d_adjScaleFactor->get_value ();
        radfac = d_adjRadFac->get_value (); 
	smooth = d_adjSmoothFactor->get_value ();
	deltascale = d_adjDeltaScale->get_value ();

	SanityCheck::bailout ((!d_HFPreview), "d_HFPreview==NULL", "TFDialogGaussianHill::updatePreviewCallback");
	SanityCheck::bailout ((!d_HFOPreview), "d_HFOPreview==NULL", "TFDialogGaussianHill::updatePreviewCallback");
	SanityCheck::bailout ((!d_HFDPreview), "d_HFOPreview==NULL", "TFDialogGaussianHill::updatePreviewCallback");
	this->previewUpToDate ();
	d_HFPreview->restore ();
	d_HFOPreview->gaussianHill (cx, cy, radius, radfac, sf, smooth, deltascale);
	d_HFDPreview->setColormap (d_HFD->getColormap());
	d_HFDPreview->draw ();
	d_cx = (int)(cx*(float)d_drawArea->width()); 
	d_cy = (int)(cy*(float)d_drawArea->height()); 
	this->drawCrosshairs (d_cx, d_cy);
}


/*
 *  daButtonPressedCallback: update the preview d_drawArea and adjustment 
 * 	the appropriate widgets (done when the user clicks on the preview)
 */
int TFDialogGaussianHill::daButtonPressedCallback (GdkEventButton *e)
{
	if (!d_cbUsePreview.get_active())
		return 1;

	int	w = this->d_drawArea->width(),
		h = this->d_drawArea->height();

	if (e->button == 1)
		{
		d_adjCX->set_value (e->x/w);
		d_adjCY->set_value (e->y/h);
		}
	else
	if (e->button == 2)
		{
		float	d = MathTrig::distance (d_adjCX->get_value()*w, 
				d_adjCY->get_value()*h, e->x, e->y);

		d/=((d_drawArea->width()+d_drawArea->height())/2);
		if (d > 1)
			d=1;
		d_adjRadius->set_value (d); 
		}
	updatePreviewCallback ();
	return 0;
}

